(use-package :fastcgi)

(defclass request-factory ()
  ())

(defclass request (fastcgi-request)
  ((parts :accessor request-parts
	  :initform 0)
   (blocks :accessor request-blocks
	   :initform nil)))

(defmethod make-fastcgi-request ((request-factory request-factory))
  (make-instance 'request))

(defclass multipart-handler (fastcgi-multipart-handler)
  ())

(defmethod fastcgi-request-params-end ((request request))
  (if (string= (caar (fastcgi-request-content-type request)) "multipart")
      (setf (fastcgi-request-stdin-handler request)
	    (make-instance 'multipart-handler :request request))))

(defmethod mime-handle-data ((handler multipart-handler) buffer start end)
  (when (string= (fastcgi-multipart-current-name handler) "kala")
    (push (subseq buffer start end)
	  (request-blocks (fastcgi-multipart-request handler)))))

(defmethod mime-handle-body-end ((handler multipart-handler))
  (incf (request-parts (fastcgi-multipart-request handler))))

(defmethod mime-handle-multipart-end ((handler multipart-handler))
  (let ((request (fastcgi-multipart-request handler)))
    (let ((o (fastcgi-request-output-stream request)))
      (format o "Content-type: text/html~A"
	      (coerce '(#\Return #\Linefeed #\Return #\Linefeed) 'string))
      (format o "<html><head><title>Multipart</title></head><body>")
      (format o "Parts: ~A~%" (request-parts request))
      (format o "Kala: ~A~%" (reverse (request-blocks request)))
      (format o "</body></html>")
      (close o))
    (fastcgi-send-end-request request 0 :request-complete)))

(defmethod fastcgi-request-stdin ((request request)
				  buffer start position)
  (when (= start position)
    (fastcgi-request-stdout request "Content-type: text/html")
    (fastcgi-request-stdout request (coerce '(#\Return #\Linefeed)
						    'string))
    (fastcgi-request-stdout request (coerce '(#\Return #\Linefeed)
						    'string))
    (let ((o (fastcgi-request-output-stream request)))
      (format o "<html><head><title>Kala</title></head><body><p>")
      (format o "~A ~A" (fastcgi-request-uri request)
	      (fastcgi-request-content-type request))
      (format o "<ul>")
      (loop for (name . value) in (fastcgi-request-params request)
	    do (format o "<li>~A, ~A</li>" name (map 'string #'code-char value)))
      (format o "</ul></body></html>")
      (close o))
    (fastcgi-send-end-request request 0 :request-complete)))

(defun main ()
  (profile:profile fastcgi::send-fastcgi-packet)
  (profile:profile fastcgi::make-packet)
  (profile:profile fastcgi::handle-fastcgi-packet-fragment)
;  (fastcgi-dynamic-server (make-instance 'request-factory))
  (fastcgi-external-unix-server "/tmp/fcgi/appserv.socket"
				(make-instance 'request-factory)))
